#include "memory.h"
#include "memory_map.h"



/* Global memory allocator. */

extern Allocator allocator;



/* Initialise the memory pool with an optional 'limit' in pages. */

Memory::Memory(unsigned int limit, offset_t region_size)
: _limit(limit), _region_size(region_size)
{
    _limited = true;
}

Memory::Memory()
: _region_size(PAGE_SIZE)
{
    _limited = false;
}

/* Allocate a new region of the given 'size' rounded to the nearest page. */

Region *Memory::region(offset_t size)
{
    std::lock_guard<std::mutex> guard(_lock);

    offset_t rounded = round(size, PAGE_SIZE);
    offset_t pages = rounded / PAGE_SIZE;

    /* Check for sufficient pages. */

    if (!_limited || (_limit >= pages))
    {
        /* Attempt to allocate aligned memory. */

        Region *region = allocator.region(rounded);

        if (region == NULL)
            return NULL;

        if (_limited)
            _limit -= pages;

        return region;
    }

    /* Return no region without sufficient pages. */

    else
        return NULL;
}

Region *Memory::region()
{
    return region(_region_size);
}

/* Release the allocated 'region'. */

void Memory::release(Region *region)
{
    std::lock_guard<std::mutex> guard(_lock);

    offset_t rounded = round(region->size(), PAGE_SIZE);
    offset_t pages = rounded / PAGE_SIZE;

    if (_limited)
        _limit += pages;

    allocator.remove(region);

    delete region;
}

// vim: tabstop=4 expandtab shiftwidth=4
